
//wxHatch CVS $Id: lchild.cpp,v 1.22 2004/05/17 12:28:42 biol75 Exp $
//part of the wxhatch project; copyright Chris Elliott
//and the University of York 2001
//NO WARRANTY
//Licenced under wxWidgets Licence; commercail redisrtibution forbidden
//
// ===========================================================================
// declarations
// ===========================================================================

// ---------------------------------------------------------------------------
// headers
// ---------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#include "wx/mdi.h"
#endif

#include <wx/docview.h>
#include <wx/textfile.h>
#include <wx/html/helpctrl.h>
#include <wx/listctrl.h>
#include <wx/treectrl.h>
#include <wx/clipbrd.h>
#include <wx/dataobj.h>


#include "projdefs.h"
#include "bitmenu.h"
#include "wx/stc/stc.h"
#include "mytext.h"
#include "lchild.h"
#include "trchild.h"
#include "dchild.h"
#include "wxhatch.h"

// Note that MDI_NEW_WINDOW and MDI_ABOUT commands get passed
// to the parent window for processing, so no need to
// duplicate event handlers here.
BEGIN_EVENT_TABLE(ListChild, wxMDIChildFrame)
EVT_MENU(wxH_CHILD_QUIT, ListChild::OnQuit)
//    EVT_MENU(MDI_REFRESH, ListChild::OnRefresh)
EVT_ACTIVATE(ListChild::OnActivate)
EVT_SIZE(ListChild::OnSize)
//    EVT_MOVE(ListChild::OnMove)
EVT_LISTBOX_DCLICK(-1, ListChild::OnListboxDClick)
EVT_CLOSE(ListChild::OnClose)
END_EVENT_TABLE()

// ---------------------------------------------------------------------------
// ListChild
// ---------------------------------------------------------------------------

ListChild::ListChild(wxMDIParentFrame *parent, const wxString& title,
const wxPoint& pos, const wxSize& size,
const long style)
: wxMDIChildFrame(parent, -1, title, pos, size, style)
{

    frame = (MyFrame *) parent ;
    ListBox = new wxListBox (this, -1, wxDefaultPosition, wxDefaultSize, 0, NULL, 
        wxLB_SINGLE | wxLB_ALWAYS_SB | wxLB_HSCROLL, wxDefaultValidator, wxT("List Box"));

    ListBox ->  SetFont (frame->myFont);
}


ListChild::~ListChild()
{
    //    frame -> my_children.DeleteObject(this);
}

bool ListChild::SaveFile (const wxString & s)
{
    if ( wxFile::Exists(s) ) wxRemoveFile(s) ;
    wxTextFile outfile (s);
    if (! outfile.Create() ) return FALSE;
    wxString sTmp = wxT("Compiler output from project ") + frame -> sName ;

    wxDateTime now = wxDateTime::Now();
    sTmp << wxT(" saved at ") << now.FormatISODate() << space << now.FormatISOTime() ;
    outfile.AddLine(sTmp);

    for (int i = 0; i < ListBox -> GetCount() ; i ++)
    {
        outfile.AddLine(ListBox->GetString(i));
    }
    if (! outfile.Write() ) return FALSE ;
    return outfile.Close();
}

void ListChild::OnQuit(wxCommandEvent& WXUNUSED(event))
{
    Close(TRUE);
}


void ListChild::OnRefresh(wxCommandEvent& WXUNUSED(event))
{
    //    if ( canvas )
    //        canvas->Refresh();
}



void ListChild::OnActivate(wxActivateEvent& event)
{
    if ( event.GetActive() )     //&& canvas )
        ListBox->SetFocus();
}


void ListChild::OnSize(wxSizeEvent& event)
{
    //    ListBox->SetSize (GetClientSize()) ;
    event.Skip();
}


void ListChild::OnClose(wxCloseEvent& event)
{
    if (this == frame -> pOutputWin) frame -> pOutputWin= NULL ;
    event.Skip();

}

void ListChild::OnCopy (wxCommandEvent & WXUNUSED(event))
{
// Write some text to the clipboard
    if (wxTheClipboard->Open())
    {
        // This data objects are held by the clipboard, 
        // so do not delete them in the app.
        wxTheClipboard->SetData( new wxTextDataObject(ListBox->GetStringSelection()) );
        wxTheClipboard->Close();
    }
}

void ListChild::OnListboxDClick(wxCommandEvent& WXUNUSED(event))
{
    DoGetError ();    
}

bool ListChild::DoGetError ()
{
    frame -> SetStatusText(wxEmptyString,1);
    wxString sSelected = ListBox->GetStringSelection();
    wxString sFilename, sLineNo, sErrorMessage, sMakefileEnding;
    bool bLineNotParsed = TRUE ;
    if (frame -> Is25()) 
        {
        sMakefileEnding = frame -> Make25FileEndings[frame -> iCompiler] ;
        }
    else
        {
        sMakefileEnding = frame -> Make25FileEndings[frame -> iCompiler] ;
        }  
    
    //error messages look like this 
    //gcc          file.cpp:lineno:error message
    //Borland      Error Exxxx file.ext lineno: error message
    //VC6          file.cpp(lineno) : error C2102: '&' requires l-value
    //Watcom       file.cpp(lineno) : Error! E006: col(21) syntax error; probable cause: missing ';'
    //D Mars       file.cpp(lineno) : Error: need explicit cast

    
    //Borland
    if (sMakefileEnding.Contains (wxT("b32")) || sMakefileEnding.Contains (wxT("bcc")))
    {
    //ignore lines which aren't errors
    if ((!sSelected.StartsWith(wxT("Error"))) && (!sSelected.StartsWith(wxT("Warning"))) )
    {
        frame -> SetStatusText(wxT("Not an error message"),1);
        return FALSE;
    }
    int iStart = 0 ;
    if (sSelected.StartsWith(wxT("Error"))) iStart = 12 ;
    if (sSelected.StartsWith(wxT("Warning"))) iStart = 14 ;
    // copy the first 12 or 14 characters to the message and then delete them
    sErrorMessage = sSelected.Left(iStart) + wxT(" ") + sSelected.AfterFirst(wxT(':'));
    sSelected = sSelected.Mid(iStart) ;
    sFilename = sSelected.BeforeFirst(wxT(' ')) ;    
    sLineNo = sSelected.AfterFirst(wxT(' ')).BeforeFirst(wxT(':')) ;
    bLineNotParsed = FALSE ;
    }

    // gcc
    if ( sMakefileEnding.IsEmpty()
      || sMakefileEnding.Contains(wxT("unx")))
    {
    //ignore lines which aren't errors
    if (sSelected.Freq(wxT(':')) < 2)
    {
        frame -> SetStatusText(wxT("Not a gcc error message"),1);
        return FALSE;
    }
    sFilename = sSelected.BeforeFirst(wxT(':')) ;
    sErrorMessage = sSelected.AfterLast(wxT(':')) ;
    sLineNo = sSelected.AfterFirst(wxT(':')).BeforeFirst(wxT(':')) ; 

    bLineNotParsed = FALSE ;
    }

    if ( (sMakefileEnding.Contains (wxT("vc")) )
      || (sMakefileEnding.Contains (wxT("wat")))
      || (sMakefileEnding.Contains (wxT("dmc")))
      || (sMakefileEnding.Contains (wxT("sc"))) )
    {   
    //VC6  Watcom DMC
    //file.cpp(lineno) : error C2102: '&' requires l-value

    //ignore lines which aren't errors
    if (sSelected.Freq(wxT(':')) < 2)
    {
        frame -> SetStatusText(wxT("Not an error message"),1);
        return FALSE;
    }
    sFilename = sSelected.BeforeFirst(wxT('(')) ;
    sErrorMessage = sSelected.AfterFirst(wxT(':')) ;
    sLineNo = sSelected.AfterFirst(wxT('(')).BeforeFirst(wxT(')')) ; 

    bLineNotParsed = FALSE ;
    }

    if (bLineNotParsed)
        {
            frame -> SetStatusText(wxT("Unknown Compiler; ") + frame -> compilerStrings [frame -> iCompiler] + wxT(" expected ; plase modifiy lchild.cpp and rebuild"),1);
            return FALSE ;
        }


    // get the line no as long instead of string
    long lineno ;
    if (!sLineNo.ToLong(&lineno))
    {
        frame -> SetStatusText(wxT("Error converting line number ") + sLineNo + wxT(" to integer format"),1);
        return FALSE;
    };

    frame->FindError ( sFilename, lineno, sErrorMessage );
    return TRUE ;
}
