/////////////////////////////////////////////////////////////////////////////
// Name:        choicdgg.cpp
// Purpose:     Choice dialogs
// Author:      Julian Smart
// Modified by: 03.11.00: VZ to add wxArrayString and multiple sel functions
// Created:     04/01/98
// RCS-ID:      $Id: mcdlg.cpp,v 1.4 2004/03/20 09:36:16 biol75 Exp $
// Copyright:   (c) wxWidgets team
// Licence:     wxWidgets license
/////////////////////////////////////////////////////////////////////////////

// ============================================================================
// declarations
// ============================================================================

#ifdef __GNUG__
    #pragma implementation "choicdgg.h"
#endif

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/dialog.h"

#endif

#if wxUSE_STATLINE
    #include "wx/statline.h"
#endif

#include "wx/dynarray.h" 
#include <wx/checkbox.h>
#include <wx/sizer.h>
#include <wx/button.h>

#include "mcdlg.h"

BEGIN_EVENT_TABLE(wxMultipleChoiceDialog, wxDialog)
EVT_BUTTON(-1, wxMultipleChoiceDialog::OnButton)
END_EVENT_TABLE()                               

wxMultipleChoiceDialog::wxMultipleChoiceDialog(wxWindow *parent,
                const wxString & caption,
                const bool * SetChecked, 
                int n, 
                const wxString * choices, 
                const bool * SetEnabled /*= NULL*/,
                long style /*= wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER */) 
    {
    	Create (parent, caption, SetChecked, n, choices, SetEnabled, style ) ;
    }; 

wxMultipleChoiceDialog::wxMultipleChoiceDialog (wxWindow *parent,
                const wxString & caption,
                wxArrayInt & SetChecked,
                int n,
                const wxArrayString& choices,
                wxArrayInt * SetEnabled /* = NULL*/,
                long style /*= wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER */)
{

    int nCount, i ;

    nCount = choices.GetCount();
    wxString * newchoices = new wxString[nCount];
    for ( i = 0; i < nCount; i++ )
    {
        newchoices[i] = choices.Item(i);
    }

    nCount = SetChecked.GetCount();
    bool * newSetChecked = new bool [nCount];
    for ( i = 0; i < nCount; i++ )
    {
        newSetChecked[i] = SetChecked[i];
    }

    nCount = SetEnabled->GetCount();
    bool * newSetEnabled = new bool [nCount];
    for ( i = 0; i < nCount; i++ )
    {
        newSetEnabled[i] = SetEnabled->Item(i);
    }

    Create (parent, caption, newSetChecked, n, newchoices, newSetEnabled, style ) ;

    delete [] newchoices ;
    delete [] newSetChecked ;
    delete [] newSetEnabled ;

}

void wxMultipleChoiceDialog::Create( wxWindow *parent,
                                const wxString& caption,
                                const bool * SetChecked,
                                int n,
                                const wxString * choices,
                                const bool * SetEnabled, /*= NULL*/
                                long style /* = wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER */)

{
    if ( !wxDialog::Create(parent, -1, caption, wxDefaultPosition, wxDefaultSize,
                           wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER) )
             return ;

    m_count = 0;
    m_topsizer = 0;

    if (n > 0) SetSelection ( SetChecked, n, choices, SetEnabled );

    wxBoxSizer *button_sizer = new wxBoxSizer( wxHORIZONTAL );
    wxButton * btnOK = new wxButton( this, wxID_OK, wxT("&OK") ) ;

    button_sizer->Add( btnOK, 0, wxALL, 10 );
    button_sizer->Add( new wxButton( this, wxID_CANCEL, wxT("Cancel") ), 0, wxALL, 10 );

    // Dialog style wxDIALOG_EX_CONTEXTHELP, in MSW may have a ?, but always add the button
    if (style | wxDIALOG_EX_CONTEXTHELP)
        button_sizer->Add( new wxButton( this, wxID_HELP_CONTEXT, wxT("&Help") ), 0, wxALL, 10 );

    m_topsizer->Add( button_sizer, 0, wxALIGN_CENTER );

    SetAutoLayout( TRUE );
    SetSizer( m_topsizer );

    m_topsizer->Fit( this );
    m_topsizer->SetSizeHints( this );

    btnOK->SetFocus();
    btnOK->SetDefault();
}


                                 
void wxMultipleChoiceDialog::SetSelection (const bool * SetChecked, int n, const wxString * choices, const bool * SetEnabled /* = NULL*/)
{
    wxCheckBox * pBox ;
    int i ;

    for ( i=0; i < m_count; i++)
    //delete any existing checkboxes 
    {        
        pBox = (wxCheckBox *) FindWindow(i+1);
        pBox -> Destroy();
    }
    m_count = n ;
    m_checked.Alloc(n) ;
    m_enabled.Alloc(n) ;

    if (!m_topsizer) m_topsizer = new wxBoxSizer( wxVERTICAL );

    for ( i = 0; i < n ; i ++)
    {
        m_checked.Add (SetChecked [i]);

        pBox = new wxCheckBox (this,  i+1, choices[i], wxPoint (10,i*10) );
        m_topsizer->Add(pBox, 1, wxEXPAND | wxALL, 10 );
        pBox -> SetValue  ( m_checked [i] ) ;
        if (SetEnabled != NULL)
        {
            m_enabled.Add ( SetEnabled [i] );
            pBox -> Enable ( m_enabled [i] );
        }
        else m_enabled.Add (TRUE);
    }
}


void wxMultipleChoiceDialog::OnButton(wxCommandEvent& event)
{
    int iID = event.GetId() ;
    if (iID == wxID_HELP_CONTEXT)
    {
        //stop dialog closeing
        event.Skip () ;
        if (GetParent()) GetParent()->ProcessEvent(event);
        return ;
    }

    if (iID == wxID_OK )
    {
        wxCheckBox * pBox ;
        for (long  i= 0; i < m_count; i ++)
        {
            pBox = (wxCheckBox *) FindWindow(i+1);
            m_checked [i] = pBox -> GetValue () ;
            m_enabled [i] = pBox -> IsEnabled ();
        }
    }

    EndModal (iID) ;
}


                                 
int wxMultipleChoiceDialog::GetSelection (bool * GetChecked, int n /*= -1 */)
{
    if ( n != -1 && n < m_count ) m_count = n ;
    {
        for (int  i= 0; i < m_count; i ++)
        {
            GetChecked [i] = m_checked [i] && m_enabled [i] ;
        }
    }
    return m_count ;
}


int  wxMultipleChoiceDialog::GetSelection (wxArrayInt & GetChecked)
{
    {
        for (int  i= 0; i < m_count; i ++)
        {
            GetChecked [i] = m_checked [i] && m_enabled [i] ;
        }
    }
    return m_count ;
}

bool wxMultipleChoiceDialog::GetSelection (int n)
{
    if ( n >= 0  &&  n < m_count ) return m_checked[n]  && m_enabled [n];
    return FALSE;
}
