//wxHatch CVS $Id: pathcfg.cpp,v 1.7 2004/04/19 08:04:40 biol75 Exp $

//part of the wxhatch project; copyright Chris Elliott
//and the University of York 2001
//NO WARRANTY
//Licenced under wxWidgets Licence; commercial redistribution forbidden
//
// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include <wx/statline.h>

#include "pathcfg.h"
// a default Dialog
BEGIN_EVENT_TABLE(pathcfg, wxDialog)
    EVT_BUTTON(del_ButtonId, pathcfg::Ondel_ButtonEvent)
    EVT_BUTTON(new_ButtonId, pathcfg::Onnew_ButtonEvent)
    EVT_LISTBOX(listbox_id, pathcfg::OnListbox)
    EVT_RADIOBUTTON(-1, pathcfg::OnChangeReuseBox)    
END_EVENT_TABLE()

pathcfg::pathcfg(wxFrame *parent, const wxArrayString & asOldPaths, const wxArrayInt & iOldState, 
        int iCompiler, const wxString & sCompiler)
            : wxDialog(parent, -1, wxT( "Add directories to wxHatch path..." ), wxDefaultPosition, wxDefaultSize,
                   wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER)
{
    mainsizer = new wxBoxSizer( wxVERTICAL);
    
	wxString * pStrings = asOldPaths.GetStringArray() ;
    pListBox = new wxListBox (this, listbox_id, wxDefaultPosition, wxDefaultSize, 
         asOldPaths.GetCount(), pStrings, wxLB_SINGLE | wxLB_SORT ) ;
	delete [] pStrings ;
    
    iReuse = iOldState ;
    iCompilerIndex = iCompiler ;    
        
    mainsizer -> Add(pListBox, 0, wxALL | wxEXPAND, 5);    
    
    reuse_StaticText = new wxStaticText(this, -1, wxT("Reuse this path..."));
    mainsizer -> Add(reuse_StaticText, 0, wxALL | wxEXPAND, 5);    

    check_BoxSizer = new wxBoxSizer( wxVERTICAL );
    
    all_RadioButton = new wxRadioButton(this, -1, wxT("Always") );
    check_BoxSizer -> Add(all_RadioButton, 1, wxALL | wxEXPAND, 5);

    wxString sTmp;
    if ( ! sCompiler.IsEmpty ())
    {
        sTmp =  wxT("For the ") + sCompiler + wxT(" compiler only") ;
    }
    this_RadioButton = new wxRadioButton(this, -1, sTmp );

    if ( sCompiler.IsEmpty ())
    {
        this_RadioButton->Enable(FALSE);
    }
    check_BoxSizer -> Add(this_RadioButton, 1, wxALL | wxEXPAND, 5);

    neveragain_RadioButton = new wxRadioButton(this, -1, wxT("For this session only") );
    check_BoxSizer -> Add(neveragain_RadioButton, 0, wxALL | wxEXPAND, 5);
   
    mainsizer -> Add(check_BoxSizer, 1, wxALL | wxEXPAND, 5);
    
    buttsizer = new wxBoxSizer(wxHORIZONTAL); // or wxHORIZONTAL
    // add controls here to buttsizer
    new_Button = new wxButton(this, new_ButtonId, wxT("Add New Path"));
    del_Button = new wxButton(this, del_ButtonId, wxT( "Delete Selected Path" )  );
    buttsizer -> Add(new_Button, 1, wxALL | wxEXPAND, 5);  
    buttsizer -> Add(del_Button, 1, wxALL | wxEXPAND, 5);
    
    mainsizer -> Add(buttsizer, 0, wxALL | wxEXPAND, 5);

    horiz_line = new wxStaticLine(this, -1);   
    mainsizer -> Add(horiz_line, 0, wxALL | wxEXPAND, 5);

    mainsizer -> Add (CreateButtonSizer(  wxOK|wxCANCEL ), 0, wxCENTER|wxALL, 10 );
    SetSizer( mainsizer );      // use the sizer for layout

    mainsizer->SetSizeHints( this );   // set size hints to honour minimum size

    if (asOldPaths.GetCount() > 0)
    {
        DoListbox (0) ;
    }
}



void pathcfg::Onnew_ButtonEvent ( wxCommandEvent & event  ) 
{  
    wxString myDir =pListBox -> GetStringSelection ();
    wxDirDialog MyDialog(this, wxT("Choose new directory to add..."), myDir);
    if (MyDialog.ShowModal() == wxID_OK)
    {
        myDir = MyDialog.GetPath() ;
        pListBox -> Append (myDir);
        int iSel = pListBox -> FindString (myDir);
        pListBox -> SetSelection (iSel ) ;
        iReuse.Insert(iThisSession, iSel);
        OnListbox(event);
    } 
};



void pathcfg::Ondel_ButtonEvent ( wxCommandEvent &  event  ) 
{ 
    int iSel =pListBox -> GetSelection ();
    
    if (iSel > wxNOT_FOUND)
    {
        pListBox -> Delete ( iSel ) ;
        // possible memory leak 
        iReuse.RemoveAt(iSel);
        
        if ( iSel >= pListBox -> GetCount() - 1 && iSel > 0)
        {
            iSel -- ;
        } 
        if ( pListBox -> GetCount() > 0)
        {
            pListBox -> SetSelection (iSel);
            OnListbox(event);
        }
    }
};


void pathcfg::OnListbox ( wxCommandEvent & WXUNUSED ( event )  )
{
    DoListbox (pListBox -> GetSelection ());
}


void pathcfg::DoListbox (int iSel)
{
    all_RadioButton -> SetValue (iReuse[iSel] == iAlwaysReuse );
    this_RadioButton -> SetValue (iReuse[iSel] == iCompilerIndex );
    neveragain_RadioButton -> SetValue (iReuse[iSel] == iThisSession ); 
}

void pathcfg::OnChangeReuseBox ( wxCommandEvent & WXUNUSED ( event )  )
{
    int iSel = pListBox -> GetSelection ();
    if (iSel == wxNOT_FOUND) return ;
    if ( all_RadioButton -> GetValue ()) iReuse [iSel] = iAlwaysReuse ;
    if ( this_RadioButton -> GetValue ()) iReuse [iSel] = iCompilerIndex ;
    if ( neveragain_RadioButton -> GetValue ()) iReuse [iSel] = iThisSession ;
}

