//wxHatch CVS $Id: wxhcombo.cpp,v 1.5 2004/05/03 15:04:46 biol75 Exp $
//part of the wxhatch project; copyright Chris Elliott
//and the University of York 2001,2002
//NO WARRANTY
//Licenced under wxWidgets Licence; commercail redisrtibution forbidden
//
//wxhcombo.cpp - derived from 
/////////////////////////////////////////////////////////////////////////////
// Name:        msw/combobox.cpp
// Purpose:     wxComboBox class
// Author:      Julian Smart
// Modified by:
// Created:     01/02/97
// RCS-ID:      $Id: wxhcombo.cpp,v 1.5 2004/05/03 15:04:46 biol75 Exp $
// Copyright:   (c) Julian Smart and Markus Holzem
// Licence:     wxWidgets licence
/////////////////////////////////////////////////////////////////////////////

// ============================================================================
// declarations
// ============================================================================

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

#ifdef __GNUG__
#pragma implementation "wxhcombo.h"
#endif

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#if wxUSE_COMBOBOX

#include <wx/config.h>

#include "wxhcombo.h"

// A custom combobox with history

wxHistoryComboBox::wxHistoryComboBox(wxWindow* parent,
        wxWindowID id,
        const wxString& value /*= wxEmptyString */,
        const wxPoint& pos /*= wxDefaultPosition */,
        const wxSize& size /*= wxDefaultSize */,
        int n /*= 0 */,
        const wxString choices[] /*= NULL */,
        long style /*= 0 */,
        const wxValidator& validator /*= wxDefaultValidator */,
        const wxString& name /*= wxEmptyString */,
        int HistoryLimit /*= 8 */)
{
    Create ( parent, id, value, pos, size, n, choices, style, validator, name, HistoryLimit );
};

bool wxHistoryComboBox::Create(wxWindow* parent,
        wxWindowID id,
        const wxString& value /*= wxEmptyString */,
        const wxPoint& pos /*= wxDefaultPosition */,
        const wxSize& size /*= wxDefaultSize */,
        int n /*= 0 */,
        const wxString choices[] /*= NULL */,
        long style /*= 0 */,
        const wxValidator& validator /*= wxDefaultValidator */,
        const wxString& name /*= wxEmptyString */,
        int HistoryLimit /*= 8 */)
{
    if (!wxComboBox::Create (parent, id, value, pos, size, n, choices, style, validator, name))
    return FALSE;

    if (!value.IsEmpty()) Append(value);
    //now apply the history
    Load (name, HistoryLimit) ;
    return TRUE;
}

void wxHistoryComboBox::Load (const wxString & name, int HistoryLimit)
{    
    if (name.IsEmpty())
    {
        wxFAIL_MSG(wxT("Empty string supplied as name of wxHistoryComboBox, \ncannot load history!"));
    }
    else
    {
        //Create a registry key
        wxConfig *pConfig = new wxConfig(wxT("wxHatch"));
        m_ConfigPath = wxT("/dialog/") + name ;
        pConfig->SetPath( m_ConfigPath ) ;
    
        size_t i = 0;
        wxString KeyName, ReadString ;
        KeyName.Printf (wxT("%d"),i);
        while ( ((int)i < HistoryLimit) && ( pConfig->Read (KeyName, &ReadString) ))
        {
            if (!ReadString.IsEmpty() && (FindString (ReadString) == wxNOT_FOUND)) Append (ReadString) ;
            i ++ ;
            KeyName.Printf (wxT("%d"),i);
        }

        pConfig -> Flush (FALSE);
        delete pConfig ;
        pConfig = NULL ;
    }
}

void wxHistoryComboBox::Save ()
{
    if (m_ConfigPath.IsEmpty())
    {
        wxFAIL_MSG(wxT("Empty string supplied as name of wxHistoryComboBox, \ncannot save history!"));    
    }
    else
    {
        wxConfig *pConfig = new wxConfig(wxT("wxHatch"));
        pConfig->SetPath(m_ConfigPath) ;

        // see if the current value is already in the list and delete if so...
        wxString WriteKey ;
        wxString sValue = GetValue () ;
        int index = FindString (sValue);
        if (wxNOT_FOUND != index)
        {
            Delete(index) ;
        }
        pConfig->Write (wxT("0"), sValue) ;
        int i ;
        for (i=0; i < GetCount(); i++)
        {
            WriteKey.Printf (wxT("%d"),i+1);
            pConfig->Write (WriteKey, GetString(i)) ;
        } // end of for
        delete pConfig ;
    } // end of if .. else
};

wxHistoryComboBox::~wxHistoryComboBox()
{
    if (GetParent() && ((wxDialog *)GetParent())->GetReturnCode() == wxID_OK )
    {
        Save ();
    }
}


IMPLEMENT_DYNAMIC_CLASS(wxHistoryComboBox, wxComboBox)

#endif // wxUSE_COMBOBOX
